#pragma once

#include "LLSymbolsCommonPublic.h"

/* ----------------------- global includes ----------------------- */
#include "MisraC.h"
#include <stddef.h>

/* ----------------------- global types ----------------------- */
#include "LLSymbolsDefs.h"



typedef enum _LLSymProtocol
{
	/** GDB on TCP protocol, using internal native implementation. The #LLSymServerAddress::tcp member must be filled  */
	symProtGDB_TCP = 0,
#ifdef _LLSYMBOLS_HAS_WINCOMMLIBS
	/** CommLibs usage (on Windows 32bit platform only). The #LLSymServerAddress::commString member must be filled */
	symProtCommLibs = 1,
#endif
	/** Direct access to a LLSymbolsServer instance. the #LLSymServerAddress::serverApi must be filled */
	symProtDirectAccess = 2,
} LLSymProtocol;


/* ----------------------- structures ----------------------- */
typedef struct _symProtServerGDB_TCP_t
{
	/** IP or hostname */
	TCHAR ip[60];
	/** default TCP port for GDB is 5000 */
	uint16_t port;
	/** timeout in ms */
	uint16_t timeout;
} symProtServerGDB_TCP_t;

typedef struct _LLSymServerAddress
{
	LLSymProtocol protocol;
#if !defined(SWIG)
	union
	{
		/** for symProtGDB_TCP usage: TCP connection options */
		symProtServerGDB_TCP_t tcp;
	#ifdef _LLSYMBOLS_HAS_WINCOMMLIBS
		/** for symProtCommLibs usage: CommString */
		TCHAR commString[64];
	#endif
		/** for symProtDirectAccess usage: pointer to an instance of LLSymbolsServer */
		const struct _LLSymServerApi * serverApi;
	};
	
#else
	/* compilation under SWIG: just use the symProtGDB_TCP protocol, without any union. On 64bit, a 4bytes padding is required,
	because in the original struct the serverApi member is aligned to 8bytes, and so all the union members */
	#if ARCHITECTURE_PTR_SIZE == 64
	uint8_t _padding[4];
	#endif
	/** IP or hostname */
	TCHAR ip[60];
	/** default TCP port for GDB is 5000 */
	uint16_t port;
	/** timeout in ms */
	uint16_t timeout;
#endif
} LLSymServerAddress;

/* handles are defined as pointer to opaque private structres */
#ifndef SWIG
typedef void* LLSymWorkspaceHandle;
typedef void* LLSymTagHandle;
typedef void* LLSymTagMapHandle;
typedef void* LLSymTypeHandle;
typedef void* LLSymGroupHandle;
#else
/* to allow the use of handles as OUTPUT parameters, they are declared as int instead of void* */
typedef int LLSymWorkspaceHandle;
typedef int LLSymTagHandle;
typedef int LLSymTagMapHandle;
typedef int LLSymTypeHandle;
typedef int LLSymGroupHandle;
#endif


/* constant to be used with LLSym_OpenWorkspace for local GDB connection on standard TCP port */
#ifndef SWIG
static const LLSymServerAddress GDBLocalHost = { symProtGDB_TCP, { _T("127.0.0.1"), 5000, 0 } };
#endif

/* ----------------------- public functions ----------------------- */
LLSYMBOLS_API LLSym_Init();
LLSYMBOLS_API LLSym_Shutdown();
LLSYMBOLS_API LLSym_IsInitOk();

LLSYMBOLS_API LLSym_OpenWorkspaceEx(uint8_t plc_area, const LLSymServerAddress* address, const TCHAR* targetComm, LLSymWorkspaceHandle* out_wksHandle);
LLSYMBOLS_API LLSym_OpenWorkspace(uint8_t plc_area, const LLSymServerAddress* address, LLSymWorkspaceHandle* out_wksHandle);
LLSYMBOLS_API LLSym_GetServerVersion(LLSymWorkspaceHandle wksHandle, uint8_t* out_verMajor, uint8_t* out_verMinor, uint8_t* out_verRevision, uint8_t* out_verBuild);
LLSYMBOLS_API LLSym_CloseWorkspace(LLSymWorkspaceHandle wksHandle);
LLSYMBOLS_API LLSym_IsValidWorkspace(LLSymWorkspaceHandle wksHandle);
LLSYMBOLS_API LLSym_GetSymbolTable(LLSymWorkspaceHandle wksHandle, const TCHAR* symbolTableFileName, bool_t overwrite);

LLSYMBOLS_API LLSym_GetGlobalTag(LLSymWorkspaceHandle wksHandle, const TCHAR* symbolName, LLSymTagHandle* out_tagHandle);
LLSYMBOLS_API LLSym_CloseTag(LLSymTagHandle tagHandle);
LLSYMBOLS_API LLSym_GetGlobalTags(LLSymWorkspaceHandle wksHandle, LLSymTagHandle** out_tagList, uint32_t* out_numTags);
LLSYMBOLS_API LLSym_FreeTagList(LLSymTagHandle* tagList);
LLSYMBOLS_API LLSym_GetGlobalTagsNum(LLSymWorkspaceHandle wksHandle, uint32_t* out_numTags);
LLSYMBOLS_API LLSym_FillGlobalTagList(LLSymWorkspaceHandle wksHandle, LLSymTagHandle* tagList, uint32_t numTags);

LLSYMBOLS_API LLSym_QueryGlobalTags(LLSymWorkspaceHandle wksHandle, const TCHAR* query, LLSymTagHandle** out_tagList, uint32_t* out_numTags);
LLSYMBOLS_API LLSym_QueryGlobalTagsNum(LLSymWorkspaceHandle wksHandle, const TCHAR* query, uint32_t* out_numTags);
LLSYMBOLS_API LLSym_QueryFillGlobalTagList(LLSymWorkspaceHandle wksHandle, const TCHAR* query, LLSymTagHandle* tagList, uint32_t numTags);

LLSYMBOLS_API LLSym_GetGlobalTagsMap(LLSymWorkspaceHandle wksHandle, LLSymTagMapHandle* out_mapHandle);
LLSYMBOLS_API LLSym_FreeTagMap(LLSymTagMapHandle mapHandle);
LLSYMBOLS_API LLSym_GetTagFromMap(LLSymTagMapHandle mapHandle, const TCHAR* symbolName, LLSymTagHandle* out_tagHandle);

LLSYMBOLS_API LLSym_GetTag(LLSymWorkspaceHandle wksHandle, const TCHAR* symbolName, LLSymTagHandle* out_tagHandle);

#ifndef SWIG
/*	deprecated	*/	LLSYMBOLS_API LLSym_ReadMemory(LLSymWorkspaceHandle wksHandle, llsym_addr_t address, void* out_buf, uint32_t size);
/*	deprecated	*/	LLSYMBOLS_API LLSym_WriteMemory(LLSymWorkspaceHandle wksHandle, llsym_addr_t address, const void* buf, uint32_t size);
#endif	//	SWIG
/*	deprecated	*/	LLSYMBOLS_API LLSym_GetCodeId(LLSymWorkspaceHandle wksHandle, uint32_t *outCodeId, bool_t checkWithTarget, uint16_t index, uint16_t subIndex );

LLSYMBOLS_API LLSym_GetTagAttribute(LLSymTagHandle tagHandle, const TCHAR* attrName, TCHAR* out_buf, uint16_t* bufsize);
LLSYMBOLS_API LLSym_GetTagName(LLSymTagHandle tagHandle, TCHAR* out_buf, uint16_t* bufsize);
LLSYMBOLS_API LLSym_GetTagDataType(LLSymTagHandle tagHandle, TCHAR* out_buf, uint16_t* bufsize);
LLSYMBOLS_API LLSym_GetTagDataTypeVar(LLSymTagHandle tagHandle, LLSymTyVar* out_type_var);
/*	deprecated	*/	LLSYMBOLS_API LLSym_GetTagAddress(LLSymTagHandle tagHandle, llsym_addr_t* out_address);
/*	deprecated	*/	LLSYMBOLS_API LLSym_GetTagAddressBitOffset(LLSymTagHandle tagHandle, llsym_addr_t* out_address, uint32_t* out_bit_offset);
/*	deprecated	*/	LLSYMBOLS_API LLSym_GetTagArrayAddress(LLSymTagHandle tagHandle, uint32_t idx1, uint32_t idx2, uint32_t idx3, llsym_addr_t* out_address);
/*	deprecated	*/	LLSYMBOLS_API LLSym_GetTagArrayAddressBitOffset(LLSymTagHandle tagHandle, uint32_t idx1, uint32_t idx2, uint32_t idx3, llsym_addr_t* out_address, uint32_t* out_bit_offset);
LLSYMBOLS_API LLSym_GetTagDataSize(LLSymTagHandle tagHandle, uint32_t* out_size);
LLSYMBOLS_API LLSym_GetTagDataFullSize(LLSymTagHandle tagHandle, uint32_t* out_size);
LLSYMBOLS_API LLSym_IsTagReadOnly(LLSymTagHandle tagHandle, bool_t* out_isReadOnly);
LLSYMBOLS_API LLSym_IsTagBitAddress(LLSymTagHandle tagHandle, bool_t* out_isBitAddress);
LLSYMBOLS_API LLSym_GetTagDescr(LLSymTagHandle tagHandle, TCHAR* out_buf, uint16_t* bufsize);
LLSYMBOLS_API LLSym_GetTagDims(LLSymTagHandle tagHandle, uint32_t* out_dim1, uint32_t* out_dim2, uint32_t* out_dim3 );

LLSYMBOLS_API LLSym_GetTagValue_Ptr(LLSymTagHandle tagHandle, void* out_buf, uint32_t bufsize);
LLSYMBOLS_API LLSym_SetTagValue_Ptr(LLSymTagHandle tagHandle, const void* buf, uint32_t bufsize);
LLSYMBOLS_API LLSym_GetTagArrayValue_Ptr(LLSymTagHandle tagHandle, uint32_t idx1, uint32_t idx2, uint32_t idx3, void* out_buf, uint32_t bufsize);
LLSYMBOLS_API LLSym_SetTagArrayValue_Ptr(LLSymTagHandle tagHandle, uint32_t idx1, uint32_t idx2, uint32_t idx3, const void* buf, uint32_t bufsize);
LLSYMBOLS_API LLSym_GetTagRawValue_Ptr(LLSymTagHandle tagHandle, void* out_buf, uint32_t bufsize);
LLSYMBOLS_API LLSym_SetTagRawValue_Ptr(LLSymTagHandle tagHandle, const void* buf, uint32_t bufsize);

#ifdef _LLSYMBOLS_HAS_VARIANT
LLSYMBOLS_API LLSym_GetTagValue_VARIANT(LLSymTagHandle tagHandle, VARIANT* out_value);
LLSYMBOLS_API LLSym_SetTagValue_VARIANT(LLSymTagHandle tagHandle, VARIANT value);
LLSYMBOLS_API LLSym_GetTagArrayValue_VARIANT(LLSymTagHandle tagHandle, uint32_t idx1, uint32_t idx2, uint32_t idx3, VARIANT* out_value);
LLSYMBOLS_API LLSym_SetTagArrayValue_VARIANT(LLSymTagHandle tagHandle, uint32_t idx1, uint32_t idx2, uint32_t idx3, VARIANT value);
#endif	//	_LLSYMBOLS_HAS_VARIANT

LLSYMBOLS_API LLSym_GetTagType(LLSymTagHandle tagHandle, LLSymTypeHandle * typeHandle);
LLSYMBOLS_API LLSym_CloseType(LLSymTypeHandle typeHandle);
LLSYMBOLS_API LLSym_TypeIsBasic(LLSymTypeHandle typeHandle, bool_t * out_IsBasic);
LLSYMBOLS_API LLSym_TypeIsComplex(LLSymTypeHandle typeHandle, bool_t * out_IsComplex);
LLSYMBOLS_API LLSym_TypeIsString(LLSymTypeHandle typeHandle, bool_t * out_IsString);
LLSYMBOLS_API LLSym_TypeIsArray(LLSymTypeHandle typeHandle, bool_t * out_IsArray);
LLSYMBOLS_API LLSym_TypeIsStruct(LLSymTypeHandle typeHandle, bool_t * out_IsStruct);
LLSYMBOLS_API LLSym_TypeIsFunctionBlock(LLSymTypeHandle typeHandle, bool_t * out_IsFunctionBlock);
LLSYMBOLS_API LLSym_TypeIsNumeric(LLSymTypeHandle typeHandle, bool_t * out_IsNumeric);
LLSYMBOLS_API LLSym_TypeIsSigned(LLSymTypeHandle typeHandle, bool_t * out_IsSigned);
LLSYMBOLS_API LLSym_TypeIsInteger(LLSymTypeHandle typeHandle, bool_t * out_IsInteger);
LLSYMBOLS_API LLSym_TypeGetSize(LLSymTypeHandle typeHandle, uint32_t * out_Size);
LLSYMBOLS_API LLSym_TypeGetName(LLSymTypeHandle typeHandle, TCHAR* out_buf, uint16_t* bufsize);
LLSYMBOLS_API LLSym_TypeGetNumChildren(LLSymTypeHandle typeHandle, uint32_t * out_NumChildren);
LLSYMBOLS_API LLSym_TypeGetChild(LLSymTypeHandle typeHandle, uint32_t childIndex, LLSymTypeHandle * out_typeChild);
LLSYMBOLS_API LLSym_TypeGetChildByName(LLSymTypeHandle typeHandle, const TCHAR* fieldName, LLSymTypeHandle * out_typeChild);
LLSYMBOLS_API LLSym_TypeGetChildName(LLSymTypeHandle typeHandle, uint32_t childIndex, TCHAR* out_childName, uint16_t* bufsize);
LLSYMBOLS_API LLSym_TypeGetArrayNumDimensions(LLSymTypeHandle typeHandle, uint32_t * out_numDimensions);
LLSYMBOLS_API LLSym_TypeGetArrayDimension(LLSymTypeHandle typeHandle, uint32_t idx, uint32_t * out_Dimension);
LLSYMBOLS_API LLSym_TypeGetArrayDimensions(LLSymTypeHandle typeHandle, uint32_t * out_Dimensions, uint8_t bufsize);
LLSYMBOLS_API LLSym_TypeGetProperties(LLSymTypeHandle typeHandle, uint32_t * out_Properties);

LLSYMBOLS_API LLSym_CreateGroup(LLSymWorkspaceHandle wksHandle, LLSymGroupHandle * out_groupHandle);
LLSYMBOLS_API LLSym_DeleteGroup(LLSymGroupHandle groupHandle);
LLSYMBOLS_API LLSym_AddTagToGroup(LLSymGroupHandle groupHandle, LLSymTagHandle tagHandle);
LLSYMBOLS_API LLSym_RemoveTagFromGroup(LLSymGroupHandle groupHandle, LLSymTagHandle tagHandle);
LLSYMBOLS_API LLSym_GetGroupInfo(LLSymGroupHandle groupHandle, uint32_t * out_numTags, uint32_t * out_byteSize);
LLSYMBOLS_API LLSym_ReadGroup(LLSymGroupHandle groupHandle);
LLSYMBOLS_API LLSym_WriteGroup(LLSymGroupHandle groupHandle);
LLSYMBOLS_API LLSym_GetTagValueFromGroup_Ptr(LLSymGroupHandle groupHandle, LLSymTagHandle tagHandle, void * data, uint32_t data_size);
LLSYMBOLS_API LLSym_SetTagValueIntoGroup_Ptr(LLSymGroupHandle groupHandle, LLSymTagHandle tagHandle, const void * data, uint32_t data_size);
#ifdef _LLSYMBOLS_HAS_VARIANT
LLSYMBOLS_API LLSym_GetTagValueFromGroup_VARIANT(LLSymGroupHandle groupHandle, LLSymTagHandle tagHandle, VARIANT* out_value);
LLSYMBOLS_API LLSym_SetTagValueIntoGroup_VARIANT(LLSymGroupHandle groupHandle, LLSymTagHandle tagHandle, VARIANT value);
#endif	//	_LLSYMBOLS_HAS_VARIANT

// AXEL: remember to add function names also to the LLSymbols.def file!
