#pragma once

/* management of the version components:
	- MAJOR:	a change here means a total incompability of the protocol. must match exactly between client and server, otherwise the client will not connect
	- MINOR:	a change here means that the protocol has been changed. this also correspond to a new stable branch (or a beta from trunk), so it usually a bigger change
	- REVISION:	a change here means that the protocol has been changed. this is inside the same stable branch, so it is usually a smaller change
	- BUILD:	a change here means only a bugfix in the client and/or the server, without any change in the protol

 MAJOR.MINOR.REVISION is in fact the version of the protocol. a client with same MAJOR but higher MINOR.REVISION can connect, with reserve.

 MAJOR.MINOR components also give the name to corresponding stable branch, e.g. 3.0.x.x, 3.2.x.x, etc.
 in this way it is possible to increase the protocol version inside the same stable branch, to implement smaller additions.
*/

#define LLSYMBOLS_VERSION_MAJOR    3
#define LLSYMBOLS_VERSION_MINOR    4
#define LLSYMBOLS_VERSION_REVISION 0
#define LLSYMBOLS_VERSION_BUILD    0

/**	Maximum supported symbol table version */
#define LLSYMTAB_MAX_SUPPORTED_VERSION_MAJOR	1

#define LLSYM_MAX_STRING  (260)				// massima lunghezza stringhe

typedef enum _LLSymResult
{
	/** Operation succeded */
	symResOk = 0,
	/** Not initialized: #LLSym_Init must be called */
	symResNotInited = 1,
	/** Can not initialize sockets */
	symResSocketInitError = 2,
	/** Can not load symbol table file from disk */
	symResSymbolTableLoadError = 3,
	/** An invalid handle has been passed */
	symResInvalidHandle = 4,
	/** An invalid parameter has been passed */
	symResInvalidParameter = 5,
	/** The specified symbol name has not been found */
	symResSymbolNotFound = 6,
	/** The specified attribute name has not been found */
	symResAttributeNotFound = 7,
	/** Invalid communication of physical address */
	symResInvalidAddress = 8,
	/** Error reading target memory */
	symResMemReadError = 9,
	/** An invalid buffer has been specified */
	symResInvalidBuffer = 10,
	/** Can not determine tag size */
	symResUnknownTagSize = 11,
	/** Error writing target memory */
	symResMemWriteError = 12,
	/** Timeout error in communication */
	symResTimeoutError = 13,
	/** Invalid VARIANT specified */
	symResInvalidVARIANT = 14,
	/** Invalid array index specified */
	symResInvalidArrayIndex = 15,
	/** Can not write a read-only tag */
	symResReadOnlyTag = 16,
	/** Connection to server lost */
	symResConnectionLost = 17,
	/** Operation failed */
	symResNotOk = 18,
	/** Invalid attribute value */
	symResInvalidAttributeValue = 19,
	/** Bit command required */
	symResBitCommandRequired = 20,
	/** Invalid tag size */
	symResInvalidTagSize = 21,
	/** Function not currently implemented */
	symResNotImplemented = 22,
	/** Symbol table version not supported */
	symResVersionNotSupported = 23,
	/** Command not supported by server */
	symResCommandNotSupported = 24,
	/** Address type mismatch (32bit vs 64bit) */
	symResAddressMismatch32vs64 = 25,
	/** Can not allocate memory */
	symResMemoryAllocError = 26,
	/** Invalid PLC area specified */
	symResInvalidArea = 27,
	/** Specified workspace invalidated */
	symResWorkspaceInvalidated = 28,
	/** Too workspaces allocated */
	symResMaxReached = 29,
	/** Buffer too small */
	symResBufferOverflow = 30,
	/** Unknown command sent to server */
	symResUnkownCommand = 31,
	/** Shutting down, command refused */
	symResShutdownInProgress = 32,
	/** Invalid communication protocol */
	symResInvalidComm = 33,
	/**	Server version is not compatible with client */
	symResServerNotCompatible = 34,		
	/**	Server version is compatible but some commands maybe not supported */
	symResOkServerNotFullSupport = 35,
	/**	File already exists */
	symResFileAlreadyExists = 36,
	/**	Cannot write file */
	symResCannotWriteFile = 37,
	/**	Cannot open file */
	symResCannotOpenFile = 38,
	/**	Cannot read file */
	symResCannotReadFile = 39,
	/** Error in segmentation of command or response */
	symResSegmentationError = 40,
	/** Duplicated handle (tag already in the group) */
	symResHandleAlreadyExists = 41,
	/** Invalid group size */
	symResInvalidGroupSize = 42,
	/** Group must be read at least once before to get tag value */
	symResGroupMustBeRead = 43,
	/** Group config error */
	symResGroupConfigError = 44,
	/** Group tag is no more valid */
	symResInvalidGroupTag = 45,
	/** Group has no tags */
	symResGroupIsEmpty = 46,
	/** Invalid tag type */
	symResInvalidTagType = 47,
	/** Transmission on socket failed */
	symResSocketError = 48,
	/** Protocol error */
	symResProtocolError = 49,
	/** Can not load symbol table file from remote target */
	symResSymbolTableUploadError = 50,
	/** Symbol table not available on the target */
	symResSymbolTableUploadNotAvailable = 51,
	/** Symbol table upload mode not supported */
	symResSymbolTableUploadModeNotSupported = 52,
	/** Symbol table upload unzip.exe missing */
	symResSymbolTableUploadUnzipNotFound = 53,
	/**	Cannot unzip simbol table */
	symResSymbolTableUploadUnzipError = 54,
	/** Command not supported by remote target */
	symResRemoteCommandNotSupported = 55,
	/** Command error by remote target */
	symResRemoteCommandError = 56,
	/** Invalid communication string */
	symResInvalidCommString = 57,
	/** Unsupported 64bit target */
	symResUnsupported64bitTarget = 58,
} LLSymResult;

/** Result type for #LLSym_TypeGetProperties */
typedef enum _LLSymTypeProperty
{
	/** Type is basic (simple scalar IEC type) */
	LLSymTypeIsBasic =			0x00000001,
	/** Type is complex (array, struct, function block) */
	LLSymTypeIsComplex =		0x00000002,
	/** Type is string */
	LLSymTypeIsString =			0x00000004,
	/** Type is array */
	LLSymTypeIsArray =			0x00000008,
	/** Type is struct */
	LLSymTypeIsStruct =			0x00000010,
	/** Type is function block */
	LLSymTypeIsFunctionBlock =	0x00000020,
	/** Type is numeric (integer or float) */
	LLSymTypeIsNumeric =		0x00000040,
	/** Type is signed */
	LLSymTypeIsSigned =			0x00000080,
	/** Type is integer */
	LLSymTypeIsInteger =		0x00000100,
}LLSymTypeProperty;

typedef uint32_t addr32_t;
typedef uint64_t addr64_t;

#ifndef LLSYM_TARGET64
	#if (defined(WIN32) && !defined(WINCE)) || ARCHITECTURE_PTR_SIZE == 64
		// on windows platform (32bit and 64bit) and all 64bit native platforms LLSymbols is always built with 64bit compatibility
		#define LLSYM_TARGET64
	#endif
#endif

#if defined(LLSYM_TARGET64)
	// support for 64bit target addresses
	typedef addr64_t llsym_addr_t;
#else
	// target 32bit addresses only (default for retrocompatibility)
	typedef addr32_t llsym_addr_t;
#endif


/** Result type for #LLSym_GetTagDataTypeVar */
typedef enum _LLSymTyVar
{
	/** Type not defined or no valid type */
	LLSymTyVarNull 		= 0,
	/** BOOL IEC Type*/
	LLSymTyVarBool 		= 1,
	/** SINT IEC Type*/
	LLSymTyVarSInt 		= 2,
	/** USINT IEC Type*/
	LLSymTyVarUSInt 	= 3,
	/** BYTE IEC Type*/
	LLSymTyVarByte 		= 4,
	/** INT IEC Type*/
	LLSymTyVarInt		= 5,
	/** UINT IEC Type*/
	LLSymTyVarUInt		= 6,
	/** WORD IEC Type*/
	LLSymTyVarWord		= 7,
	/** DINT IEC Type*/
	LLSymTyVarDInt		= 9,
	/** UDINT IEC Type*/
	LLSymTyVarUDInt		= 10,
	/** DWORD IEC Type*/
	LLSymTyVarDWord		= 11,
	/** LINT IEC Type*/
	LLSymTyVarLInt		= 12,
	/** ULINT IEC Type*/
	LLSymTyVarULInt		= 13,
	/** LWORD IEC Type*/
	LLSymTyVarLWord		= 14,
	/** REAL IEC Type*/
	LLSymTyVarReal		= 15,
	/** LREAL IEC Type*/
	LLSymTyVarLReal		= 16,
	/** STRING IEC Type*/
	LLSymTyVarString	= 17,
	/** User defined IEC Type*/
	LLSymTyVarUserDef	= 18,
	/** Any pointer IEC Type*/
	LLSymTyVarPointer	= 20,
}LLSymTyVar;

