#pragma once

#ifndef LLSYM_EXTERNAL_MISRAC

#ifndef USE_STDINT_FOR_MISRA_C
#if defined(__GNUC__) || _MSC_VER >= 1800 || __STDC_VERSION__ >= 199901L
/* auto-define it for known compilers that support stdint.h */
#define USE_STDINT_FOR_MISRA_C
#endif
#endif


#if defined USE_STDINT_FOR_MISRA_C
	/* if USE_STDINT_FOR_MISRA_C is defined, <stdint.h> is used to provide standard definitions for fixed types (C99) */
	#include <stdint.h>

	/* required types not defined in stdint.h! */
	typedef char				char_t;
	typedef float				float32_t;
	typedef double				float64_t;
	typedef uint8_t 			bool_t;
	

#else
	/* compiler DOES NOT support C99, define all MisraC types. Valid for standard 32bit compilers. */
	typedef char				char_t;
	typedef unsigned char   	uint8_t;
	typedef signed char 		int8_t;
	typedef unsigned short  	uint16_t;
	typedef short   			int16_t;
	typedef unsigned long   	uint32_t;
	typedef long				int32_t;
	typedef unsigned long long	uint64_t;
	typedef long long			int64_t;
	typedef float				float32_t;
	typedef double				float64_t;
	typedef uint8_t 			bool_t;
#endif

/*	Boolean values	*/
#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

/*  Word composition/decomposition utilities	*/
#ifndef MAKE_WORD
#define MAKE_WORD(lo,hi)	((uint16_t)(lo) + ((uint16_t)(hi)<<8))
#endif

#ifndef LO_BYTE
#define LO_BYTE(b)  		(int8_t)((b) & 0xFF )
#endif

#ifndef HI_BYTE
#define HI_BYTE(b)  		(int8_t)(((uint16_t)(b) >> 8 ) & 0xFF )
#endif

/* try auto-detect 32 vs 64 bit architecture */
#ifndef ARCHITECTURE_PTR_SIZE
	#ifdef USE_STDINT_FOR_MISRA_C
		#if ((UINTPTR_MAX) == (UINT64_MAX))
			#define ARCHITECTURE_PTR_SIZE 64
		#else
			#define ARCHITECTURE_PTR_SIZE 32
		#endif
	#endif
#endif

#endif // ndef LLSYM_EXTERNAL_MISRAC
