module.exports = function(RED) {
	
	function get_tag_value(config) {
		
		// node properties:
		RED.nodes.createNode(this,config);
		this.address = config.address;
		this.symbol = config.symbol;
		this.status({});
		
		var LLSymbols = require("../../lib/LLSymbols.node");
		var shared_functions = require('../shared_functions.js');
		
		// init & open/create new workspace
		shared_functions.add_workspace(this);
		
		//get global tag
		shared_functions.get_global_tag(this);
		
		// on inject button pression
		this.on('input', function(msg) 
		{	
			if (this.tag_handle == undefined) 
				return;
			
			
			this.status({fill:"yellow",shape:"dot",text:"Processing..."});		
			
			if(this.isValid) 	
			{
				var isValidWorkspace = LLSymbols.LLSym_IsValidWorkspace_wrap(this.wks.wks_handle_object);
				if( isValidWorkspace != shared_functions.LLSymResult.symResOk )
				{
					shared_functions.invalidateAllNodes(this);
					
					shared_functions.remove_invalid_workspace(this);
					
					shared_functions.add_workspace(this);
					
					shared_functions.get_global_tag(this);
				}
				
			}
			else
			{
				shared_functions.add_workspace(this);
				
				shared_functions.get_global_tag(this);
			}
			
			// Normalmente in ogni caso il tag è valido
			if(this.isValid) 
			{
				var data_value = 0;
				switch(this.data_type) 
				{
					case "BOOL": 
					{
						data_value = LLSymbols.GetTagValue_boolean(this.tag_handle);
						break;
					}
					case "STRING": 
					{
						data_value = LLSymbols.GetTagValue_string(this.tag_handle);
						break;
					}
					default: 
					{
						data_value = LLSymbols.GetTagValue_double(this.tag_handle);
						break;
					}
				}
					
				msg.type = this.data_type;
				msg.payload = data_value;
				this.send(msg);
				
				this.status( {fill:"green",shape:"dot",text:"Success"} );
			}
			else
				this.status( {fill:"red",shape:"ring",text:"Invalid Workspace."} );
		});		
		
		
		
		// called on re-deploy
		this.on('close', function() 
		{
			
			// close global tag
			if(this.isValid)
				shared_functions.close_global_tag(this);
			
			// close workspace & shutdown
			shared_functions.remove_workspace(this);
		});	
	}
	
	
	RED.nodes.registerType('GetTagValue',get_tag_value);
}