module.exports = function(RED) {
		
	function set_tag_value(config) {
		
		// i valori corrispondono all'indice della <select>
		var VALUE_SOURCE_TYPE = Object.freeze({
			'PAYLOAD': 0, 
			'VALUE': 1 
		});
		
		// node properties:
		RED.nodes.createNode(this,config);
		this.address = config.address;
		this.symbol = config.symbol;
		this.value = config.value;
		this.valueSourceType = config.valueSourceType;
		this.status({});
		
		var LLSymbols = require("../../lib/LLSymbols.node");
		var shared_functions = require('../shared_functions.js');
		
		// init & open/create new workspace
		shared_functions.add_workspace(this);
		
		// get global tag
		shared_functions.get_global_tag(this);
		
		
		// on inject button pression
		this.on('input', function(msg) 
		{	
			if (this.tag_handle == undefined) 
				return;
			
			
			this.status({fill:"yellow",shape:"dot",text:"Processing..."});		
			
			if(this.isValid) 	
			{
				var isValidWorkspace = LLSymbols.LLSym_IsValidWorkspace_wrap(this.wks.wks_handle_object);
				if( isValidWorkspace != shared_functions.LLSymResult.symResOk )
				{
					shared_functions.invalidateAllNodes(this);
					
					shared_functions.remove_invalid_workspace(this);
					
					shared_functions.add_workspace(this);
					
					shared_functions.get_global_tag(this);
				}
				
			}
			else
			{
				shared_functions.add_workspace(this);
				
				shared_functions.get_global_tag(this);
				
			}
			
			
			// Normalmente in ogni caso il tag è valido
			if( this.isValid ) 
			{
				var data_value = 0;
				switch( this.data_type ) 
				{
					case "BOOL": 
					{
						// set tag from the value property of the module
						var log_val;
						if(this.valueSourceType == VALUE_SOURCE_TYPE.VALUE)
						{
							var bool_var = (this.value == 'true');
							log_val = bool_var;
							LLSymbols.SetTagValue_boolean(this.tag_handle, bool_var);
						}
						else
						{
							//check if msg.payload type is correct
							if(typeof msg.payload == 'boolean')
							{
								log_val = msg.payload;
								LLSymbols.SetTagValue_boolean(this.tag_handle, msg.payload);
							}
							else
								shared_functions.Thow_Error( shared_functions.Error_Type.onPayloadType, null, "boolean", this);
						}
						
						shared_functions.Log(this.symbol, log_val, this);
						break;
					}
					
					case "STRING": 
					{
						// set tag from the value property of the module
						if(this.valueSourceType == VALUE_SOURCE_TYPE.VALUE)
						{
							log_val = this.value;
							LLSymbols.SetTagValue_string(this.tag_handle, this.value);
						}
						else
						{
							//check if msg.payload type is correct
							if(typeof msg.payload == 'string')
							{
								log_val = msg.payload;
								LLSymbols.SetTagValue_string(this.tag_handle, msg.payload);
							}
							else
								shared_functions.Thow_Error( shared_functions.Error_Type.onPayloadType, null, "string", this );
						}
						
						shared_functions.Log(this.symbol, log_val, this);
						break;
					}
					
					default: 
					{
						// set tag from the value property of the module
						if(this.valueSourceType == VALUE_SOURCE_TYPE.VALUE)
						{
							log_val = this.value;
							LLSymbols.SetTagValue_double(this.tag_handle, parseFloat(this.value));
						}
						else
						{
							//check if msg.payload type is correct
							if(typeof msg.payload == 'number')
							{
								log_val = msg.payload;
								LLSymbols.SetTagValue_double(this.tag_handle, msg.payload);
							}
							else
								shared_functions.Thow_Error( shared_functions.Error_Type.onPayloadType, null, "number", this );
						}
						
						shared_functions.Log(this.symbol, log_val, this);
						break;
					}
					
				}
				
				msg.type = this.data_type;
				msg.payload = data_value;
				this.send(msg);
				
				this.status({fill:"green",shape:"dot",text:"Success"});	
			}
			else
				this.status({fill:"red",shape:"ring",text:"Invalid Workspace."});
		});		
		
		
		
		// called on re-deploy
		this.on('close', function() 
		{
			// close global tag
			if(this.isValid)
				shared_functions.close_global_tag(this);
			
			// close workspace & shutdown
			shared_functions.remove_workspace(this);		
		});	
	}
	
	
	RED.nodes.registerType('SetTagValue',set_tag_value);
}