// Replica dei valori di ritorno delle funzioni della LLSymbols: 
// per ulteriori informazioni su ciascun valore, consultare LLSymbolsDefs.h
var LLSymResult =
{
	symResOk : 0,
	symResNotInited : 1,
	symResSocketInitError : 2,
	symResSymbolTableLoadError : 3,
	symResInvalidHandle : 4,
	symResInvalidParameter : 5,
	symResSymbolNotFound : 6,
	symResAttributeNotFound : 7,
	symResInvalidAddress : 8,
	symResMemReadError : 9,
	symResInvalidBuffer : 10,
	symResUnknownTagSize : 11,
	symResMemWriteError : 12,
	symResTimeoutError : 13,
	symResInvalidVARIANT : 14,
	symResInvalidArrayIndex : 15,
	symResReadOnlyTag : 16,
	symResConnectionLost : 17,
	symResNotOk : 18,
	symResInvalidAttributeValue : 19,
	symResBitCommandRequired : 20,
	symResInvalidTagSize : 21,
	symResNotImplemented : 22,
	symResVersionNotSupported : 23,
	symResCommandNotSupported : 24,
	symResAddressMismatch32vs64 : 25,
	symResMemoryAllocError : 26,
	symResInvalidArea : 27,
	symResWorkspaceInvalidated : 28,
	symResMaxReached : 29,
	symResBufferOverflow : 30,
	symResUnkownCommand : 31,
	symResShutdownInProgress : 32,
	symResInvalidComm : 33,
	symResServerNotCompatible : 34,		
	symResOkServerNotFullSupport : 35,
	symResFileAlreadyExists : 36,
	symResCannotWriteFile : 37,
	symResCannotOpenFile : 38,
	symResCannotReadFile : 39,
	symResSegmentationError : 40,
	symResHandleAlreadyExists : 41,
	symResInvalidGroupSize : 42,
	symResGroupMustBeRead : 43,
	symResGroupConfigError : 44,
	symResInvalidGroupTag : 45,
	symResGroupIsEmpty : 46,
	symResInvalidTagType : 47,
	symResSocketError : 48,
	symResProtocolError : 49,
	symResSymbolTableUploadError : 50,
	symResSymbolTableUploadNotAvailable : 51,
	symResSymbolTableUploadModeNotSupported : 52,
	symResSymbolTableUploadUnzipNotFound : 53,
	symResSymbolTableUploadUnzipError : 54,
	symResRemoteCommandNotSupported : 55,
	symResRemoteCommandError : 56,
	symResInvalidCommString : 57,
	symResUnsupported64bitTarget : 58
};

var Error_Type =
{
	onParamsNum : 1,			// Errore sul numero di parametri
	onPayloadType : 2,			// Errore sul tipo del payload (serve alla SetTagValue)
	onLLSymbolsFunction : 3		// Errore specifico di una funzione della LLSymbols 
};

var LLSymbols = require("../lib/LLSymbols.node");

/**
 *  Workspace opening procedure, it adds a new workspace or 
 *  takes an existing one
 */
module.exports.add_workspace = function(node) 
{
	var context = node.context();

	// Check if all the fields are set or not
	if( node.symbol == '' )
		Thow_Error( Error_Type.onParamsNum, null, null, node );
		
	var wks_handles_array = context.global.get('wks_handles_array');
	if (!wks_handles_array)
	{
		// la prima volta se l'array globale non esiste lo crea vuoto e lo mette nel contesto globale
		wks_handles_array = [];
		context.global.set('wks_handles_array', wks_handles_array);
	}
	var create_new_wks = true;
		
	// Check if there is an existing workspace:
	for(i = 0; i < wks_handles_array.length; i++)
	{
		var wks_tmp = wks_handles_array[i];
		if(wks_tmp.address == node.address)
		{
			// A workspace matches
			node.wks = wks_tmp;		// workspace handle
			wks_tmp.counter++;
			wks_tmp.binded_nodes.push(node);
			create_new_wks = false;
			break;
		}
	}
		
	// Otherwise create a new workspace:
	if( create_new_wks )
	{
		// host
		var ip_address = "";
		var port = 0;

		// il campo address può essere solo IP (porta default GDB 5000) oppure IP:PORTA
		if (node.address.indexOf(":") != -1)
		{
			var arr = node.address.split(":");
			ip_address = arr[0];
			port = parseInt( arr[1] );				
		}
		else
		{
			ip_address = node.address;
			port = 5000;				
		}
			
		if( wks_handles_array.length == 0 )
		{
			/* INIT solo alla creazione della prima workspace */
			var return_val = LLSymbols.LLSym_Init_wrap();
			if( return_val != LLSymResult.symResOk ) 
				Thow_Error( Error_Type.onLLSymbolsFunction, "LLSym_Init", return_val, node );
				
			node.log("Init function invoked.");//
		}
			
		/* OPEN WORKSPACE */
		node.log("Opening Workspace...");//
		var wks_result = LLSymbols.LLSym_OpenWorkspace_wrap(0, ip_address, port);
		var return_val = wks_result[0];					// returned value
		if( return_val != LLSymResult.symResOk && return_val != LLSymResult.symResOkServerNotFullSupport ) 
			Thow_Error( Error_Type.onLLSymbolsFunction, "LLSym_OpenWorkspace", return_val, node );
			
		node.log("Workspace opened.");//
		
		
		// add it to the array
		var workspace = 
		{
			address: 			node.address, 
			counter: 			1,
			wks_handle_object:  wks_result[1],
			binded_nodes:		[]
		};
		
		node.wks = workspace;
		
		workspace.binded_nodes.push(node);
		wks_handles_array.push(workspace);
	}
}


/**
 *  Get global tag stuff function.
 */
module.exports.get_global_tag = function(node) 
{
	var context = node.context();
	
	/* GET GLOBAL TAG */
	var glob_tag_result = LLSymbols.LLSym_GetGlobalTag_wrap(node.wks.wks_handle_object, node.symbol);
	var return_val = glob_tag_result[0];	// return value
	
	if( return_val != LLSymResult.symResOk ) 
		Thow_Error( Error_Type.onLLSymbolsFunction, "LLSym_GetGlobalTag", return_val, node );
	
	
	node.tag_handle = glob_tag_result[1];	// tag handle

	node.log("Obtained global tag for symbol {"+node.symbol+"}.");//
	
	node.isValid = true;
	
	/* GET TAG VALUE */
	return_val = LLSymbols.LLSym_GetTagDataType_wrap(node.tag_handle);

	if( return_val[0] != LLSymResult.symResOk ) 
		Thow_Error( Error_Type.onLLSymbolsFunction, "LLSym_GetTagDataType", return_val[0], node );

	node.data_type = return_val[1];
}


/**
 *  Close global tag stuff function.
 */

function close_global_tag(node) 
{
	/* CLOSE TAG */
	if (node.tag_handle != undefined) 
	{
		var return_val = LLSymbols.LLSym_CloseTag_wrap(node.tag_handle);
		// I controlli sulla chiusura dei tag rendono impossibile la chiusura di una workspace quando cambia il codice scaricato da LogicLab
		// if( return_val != LLSymResult.symResOk ) 
		// 	Thow_Error( Error_Type.onLLSymbolsFunction, "LLSym_CloseTag", return_val, node );
					
		node.log("Released global tag for symbol {"+node.symbol+"}.");//
	}	
}


/**
 *  Workspace closing stuff function.
 */
module.exports.remove_workspace = function(node)
{
	var context = node.context();
	wks_handles_array = context.global.get('wks_handles_array');
			
	for(i = 0; i < wks_handles_array.length; i++)	// Check for the workspace which this node belongs to
	{
		var wks_tmp = wks_handles_array[i];
		if( wks_tmp == node.wks )						// if a workspace matches
		{
			wks_tmp.counter--;	
			if(wks_tmp.counter == 0)				// If this node is the last one for THIS workspace
			{
				wks_handles_array.splice(i, 1);
				
				/* CLOSE WORKSPACE */
				var return_val = LLSymbols.LLSym_CloseWorkspace_wrap(wks_tmp.wks_handle_object);
				
				if( return_val != LLSymResult.symResOk ) 
					node.log("Warning: LLSym_CloseWorkspace returned " + return_val);
				node.log("Workspace closed.");//
			}		
			break;
		}
	}
			
	shutdown(node);			
}


/**
 *  Invalid Workspace closing stuff function.
 */
module.exports.remove_invalid_workspace = function(node)
{
	var context = node.context();
	var wks_handles_array = context.global.get('wks_handles_array');
	
	// Check for the workspace which this node belongs to
	for(i = 0; i < wks_handles_array.length; i++)					
	{
		var wks_tmp = wks_handles_array[i];
		if(wks_tmp == node.wks)										// if a workspace matches
		{
			/* CLOSE ALL GLOBAL TAGS */
			for(j = 0; j < wks_tmp.binded_nodes.length; j++)
				close_global_tag(wks_tmp.binded_nodes[j]);
			
			wks_handles_array.splice(i, 1);
			
			/* CLOSE WORKSPACE */
			var return_val = LLSymbols.LLSym_CloseWorkspace_wrap(wks_tmp.wks_handle_object);
			
			if( return_val != LLSymResult.symResOk ) 
				node.log("Warning: LLSym_CloseWorkspace returned " + return_val);
			node.log("Workspace closed.");//
			
			wks_tmp.isClosed = true;

			/* SHUTDOWN */
			shutdown(node);	
			
			break;
		}
	}	
}


module.exports.invalidateAllNodes = function(node) 
{
	var context = node.context();
	var wks_handles_array = context.global.get('wks_handles_array');
	for(i = 0; i < wks_handles_array.length; i++)
	{
		var wks_tmp = wks_handles_array[i];
		if(wks_tmp == node.wks)						// if a workspace matches
		{
			var wks_handles_array_length = wks_handles_array[i].binded_nodes.length;
			for(j = 0; j < wks_handles_array_length; j++)
			{
				wks_handles_array[i].binded_nodes[j].isValid = false;
			}
		}
	}
	context.global.set('wks_handles_array', wks_handles_array);
}



function shutdown(node)
{
	var context = node.context();
	wks_handles_array = context.global.get('wks_handles_array');
	
	if(wks_handles_array.length == 0)		// if the node is the last one among ALL the flows
	{
		var return_val = LLSymbols.LLSym_Shutdown_wrap();
		if( return_val != LLSymResult.symResOk ) 
			node.log("Warning: LLSym_Shutdown returned " + return_val);
			// Thow_Error( Error_Type.onLLSymbolsFunction, "LLSym_Shutdown", return_val, node );
			
		node.log("Shutdown.");//
	}
}


/**
 *  Error function.
 *  It sets node status to 'error' and halts the execution.
 */
function Thow_Error( error_type, funcname, err_code, node ) 
{	
	// symResInvalidHandle è una condizione normale quando si scarica il PLC,
	// altrimenti l'errore è da gestire	
	if( err_code != LLSymResult.symResInvalidHandle ) 
	{
		var error_str;
		switch ( error_type )
		{
			case Error_Type.onParamsNum:
			{
				error_str = "ERROR: One or more properties are not set.";
				break;
			}

			case Error_Type.onPayloadType:
			{
				error_str = "ERROR: The msg.payload entry property must be a " + err_code + " type";
				break;
			}

			default:
			{
				error_str = "ERROR: " + funcname + " returned " + err_code;
			}
		}
		
		node.status( {fill:"red",shape:"ring",text:"Error"} );
		throw( error_str );
	}
}


/**
 *  Log function.
 */
module.exports.Log = function(symbol, value, node) 
{	
	node.log("Set operation: Symbol \""+symbol+"\" successfully set to value \""+value+"\".");
}

// explicitly exports some functions to be used within specific-node javascript files
module.exports.Thow_Error = Thow_Error;
module.exports.Error_Type = Error_Type;

module.exports.close_global_tag = close_global_tag;
module.exports.LLSymResult = LLSymResult;